@@ -92,6 +92,7 @@ public class AuxWindow extends JFrame {
      * @param  parent   the parent component of the new window - may be
      *         used for positioning
      */
+    @SuppressWarnings("this-escape")
     public AuxWindow( String title, Component parent ) {
         setTitle( title );
         setDefaultCloseOperation( DISPOSE_ON_CLOSE );
@@ -207,7 +208,7 @@ public class AuxWindow extends JFrame {
      * <p>
      * An ID can be supplied to indicate the page which should be shown
      * in the help viewer when context-sensitive help is requested.
-     * This may be <tt>null</tt> if no change in the help page should
+     * This may be <code>null</code> if no change in the help page should
      * be made (for instance if there is no help specific to this window).
      *
      * @param  helpID  the ID of the help item for this window
@@ -518,7 +519,7 @@ public class AuxWindow extends JFrame {
      * to the right.  The second window is repositioned relative to the
      * first one.
      * 
-     * @param   first   first window, or <tt>null</tt>
+     * @param   first   first window, or <code>null</code>
      * @param   second  second window
      */
     public static void positionAfter( Component first, Window second ) {
@@ -715,17 +716,7 @@ public class AuxWindow extends JFrame {
                 ControlWindow.getInstance().makeVisible();
             }
             else if ( this == aboutAct ) {
-          
-                Object[] msg = new Object[] {
-                    TopcatUtils.getAbout(),
-                    Box.createVerticalStrut( 10 ), 
-                    getSponsorLogos(),
-                };
-                JOptionPane.showMessageDialog( AuxWindow.this, 
-                                               msg,
-                                               "About TOPCAT",
-                                               JOptionPane.INFORMATION_MESSAGE,
-                                               ResourceIcon.getTopcatLogo() );
+                TopcatUtils.showAbout( AuxWindow.this );
             }
         }
     }
