@@ -180,8 +180,8 @@ public class ControlWindow extends AuxWindow
     private final TableColumnModelListener columnWatcher_ = this;
     private final WindowListener windowWatcher_ = new ControlWindowListener();
     private final StarTableOutput taboutput_ = new StarTableOutput();
-    private final boolean canWrite_ = Driver.canWrite();
-    private final boolean canRead_ = Driver.canRead();
+    private final boolean canWrite_ = true;
+    private final boolean canRead_ = true;
     private final TransferHandler importTransferHandler_ = 
         new ControlTransferHandler( true, false );
     private final TransferHandler exportTransferHandler_ =
@@ -540,12 +540,12 @@ public class ControlWindow extends AuxWindow
         listActs.put( delkey, removeAct_ );
         Object upkey = upAct_.getValue( Action.NAME );
         listInputs.put( KeyStroke.getKeyStroke( KeyEvent.VK_UP,
-                                                InputEvent.ALT_MASK ),
+                                                InputEvent.ALT_DOWN_MASK ),
                         upkey );
         listActs.put( upkey, upAct_ );
         Object downkey = downAct_.getValue( Action.NAME );
         listInputs.put( KeyStroke.getKeyStroke( KeyEvent.VK_DOWN,
-                                                InputEvent.ALT_MASK ),
+                                                InputEvent.ALT_DOWN_MASK ),
                         downkey );
         listActs.put( downkey, downAct_ );
 
@@ -828,11 +828,11 @@ public class ControlWindow extends AuxWindow
      *
      * @param  table  the table to add
      * @param  location  location string indicating the provenance of
-     *         <tt>table</tt> - preferably a URL or filename or something
+     *         <code>table</code> - preferably a URL or filename or something
      * @param  select  true iff the newly-added table should become the
      *         currently selected table
      * @return the newly-created TopcatModel object corresponding to 
-     *         <tt>table</tt>
+     *         <code>table</code>
      */
     public TopcatModel addTable( StarTable table, String location,
                                  boolean select ) {
@@ -1262,8 +1262,8 @@ public class ControlWindow extends AuxWindow
      *
      * @param   confirm  whether to seek confirmation from the user
      * @return  whether shutdown took place.  If the user aborted the
-     *          exit, then <tt>false</tt> will be returned.  If the exit
-     *          did happen, then either <tt>true</tt> will be returned
+     *          exit, then <code>false</code> will be returned.  If the exit
+     *          did happen, then either <code>true</code> will be returned
      *          or (standalone case) there will be no return.
      */
     public boolean exit( boolean confirm ) {
@@ -2041,7 +2041,7 @@ public class ControlWindow extends AuxWindow
          * Throw it away after that.  Not ideal. */
         TableLoadDialog tld0;
         try {
-            tld0 = tldClazz.newInstance();
+            tld0 = tldClazz.getDeclaredConstructor().newInstance();
         }
         catch ( Throwable e ) {
             logger_.log( Level.WARNING, "Can't set up TAP action", e );
