@@ -1,6 +1,5 @@
 package uk.ac.starlink.topcat;
 
-import gnu.jel.CompilationException;
 import java.awt.Component;
 import java.awt.event.ActionEvent;
 import java.awt.event.KeyEvent;
@@ -53,6 +52,7 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
     private final Action tocolAct;
     private final Action highlightAct;
     private final Action countAct;
+    private final Action editAct;
     private final Action invertAct;
     private final Action sampleAct;
     private final Action headAct;
@@ -77,6 +77,7 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
      * @param  tcModel  model containing the data for the table concerned
      * @param  parent   component used for window positioning
      */
+    @SuppressWarnings("this-escape")
     public SubsetWindow( final TopcatModel tcModel, Component parent ) {
         super( tcModel, "Row Subsets", parent );
         this.tcModel = tcModel;
@@ -105,22 +106,20 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
 
         /* Configure column widths and alignments. */
         TableColumnModel tcm = jtab.getColumnModel();
-        tcm.getColumn( tcm.getColumnIndex( CNAME_ID ) )
-           .setPreferredWidth( 64 );
+        tcm.getColumn( tcm.getColumnIndex( CNAME_ID ) ).setMaxWidth( 80 );
         tcm.getColumn( tcm.getColumnIndex( CNAME_NAME ) )
-           .setPreferredWidth( 200 );
+           .setPreferredWidth( 120 );
         tcm.getColumn( tcm.getColumnIndex( CNAME_SIZE ) )
-           .setPreferredWidth( 100 );
-        tcm.getColumn( tcm.getColumnIndex( CNAME_FRACTION ) )
-           .setPreferredWidth( 80 );
+           .setMaxWidth( 80 );
+        tcm.getColumn( tcm.getColumnIndex( CNAME_FRACTION ) ).setMaxWidth( 80 );
         tcm.getColumn( tcm.getColumnIndex( CNAME_EXPRESSION ) )
-           .setPreferredWidth( 200 );
-        tcm.getColumn( tcm.getColumnIndex( CNAME_COLID ) )
-           .setPreferredWidth( 80 );
+           .setPreferredWidth( 300 );
+        tcm.getColumn( tcm.getColumnIndex( CNAME_COLID ) ).setMaxWidth( 80 );
         DefaultTableCellRenderer rightRend = new DefaultTableCellRenderer();
         rightRend.setHorizontalAlignment( SwingConstants.RIGHT );
         tcm.getColumn( tcm.getColumnIndex( CNAME_FRACTION ) )
            .setCellRenderer( rightRend );
+        jtab.setAutoResizeMode( JTable.AUTO_RESIZE_LAST_COLUMN );
 
         /* Customise the JTable's column model to provide control over
          * which columns are displayed. */
@@ -152,13 +151,19 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
                                      "(Re)count the number of rows in each " +
                                      "subset" );
 
-        /* Action for highlighting selectd subset. */
+        /* Action for highlighting selected subset. */
         highlightAct = new SubsetAction( "Highlight subset",
                                          ResourceIcon.HIGHLIGHT,
                                          "Highlight the selected subset by "
                                        + "marking its rows in the table window "
                                        + "and ensuring it's visible in plots" );
 
+        /* Action for editing selected subset. */
+        editAct = new SubsetAction( "Edit subset", ResourceIcon.EDIT,
+                                    "Show a window that lets you edit the "
+                                  + "expression and name for "
+                                  + "the selected subset" );
+
         /* Action for producing inverse subset. */
         invertAct = new SubsetAction( "Invert subset", ResourceIcon.INVERT,
                                       "Create new subset complementary to " +
@@ -223,6 +228,7 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
                 }
                 removeAct.setEnabled( hasSelection && ! isAllSelected );
                 tocolAct.setEnabled( hasUniqueSelection );
+                editAct.setEnabled( hasUniqueSelection );
                 invertAct.setEnabled( hasUniqueSelection );
                 highlightAct.setEnabled( hasUniqueSelection );
                 if ( subsetTransmitter != null ) {
@@ -250,9 +256,6 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
                 if ( rset != null ) {
                     selectionModel.setValueIsAdjusting( true );
                     selectionModel.clearSelection();
-                    @SuppressWarnings("rawtypes")
-                    ComboBoxModel tcSelModel = 
-                        tcModel.getSubsetSelectionModel();
                     for ( int irow = 0; irow < subsetsTableModel.getRowCount();
                           irow++ ) {
                         if ( getSubset( toUnsortedIndex( irow ) ) == rset ) {
@@ -276,6 +279,7 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
         getToolBar().add( sampleAct );
         getToolBar().add( headAct );
         getToolBar().add( tailAct );
+        getToolBar().add( editAct );
         getToolBar().add( invertAct );
         getToolBar().add( classifyAct );
         getToolBar().add( removeAct );
@@ -295,6 +299,7 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
         subsetsMenu.add( sampleAct );
         subsetsMenu.add( headAct );
         subsetsMenu.add( tailAct );
+        subsetsMenu.add( editAct );
         subsetsMenu.add( invertAct );
         subsetsMenu.add( classifyAct );
         subsetsMenu.add( removeAct );
@@ -403,41 +408,17 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
                 return getSubset( irow ) instanceof SyntheticRowSubset;
             }
             public void setValue( int irow, Object value ) {
-                String expr = value.toString();
-                SyntheticRowSubset rset =
-                   (SyntheticRowSubset) getSubset( irow );
-                int rsetId = subsets.indexToId( irow );
-                if ( TopcatJELUtils
-                    .isSubsetReferenced( tcModel, rsetId, expr ) ) {
-                    String[] msg = new String[] {
-                        "Recursive subset expression disallowed:",
-                        "\"" + expr + "\"" +
-                        " directly or indirectly references subset " +
-                        rset.getName(),
-                    };
-                    JOptionPane.showMessageDialog( SubsetWindow.this, msg,
-                                                   "Expression Error",
-                                                   JOptionPane.ERROR_MESSAGE );
-                    return;
+                String expr = (String) value;
+                SubsetQueryWindow qwin =
+                    SubsetQueryWindow
+                   .editSubsetDialog( tcModel, SubsetWindow.this, irow );
+                qwin.getExpressionField().setText( expr );
+                if ( qwin.perform() ) {
+                    qwin.dispose();
                 }
-                try {
-                    rset.setExpression( value.toString(),
-                                        tcModel.createJELRowReader() );
+                else {
+                    qwin.setVisible( true );
                 }
-                catch ( CompilationException e ) {
-                    String[] msg = new String[] {
-                        "Syntax error in algebraic subset expression \""
-                        + value + "\":",
-                        e.getMessage(),
-                    };
-                    JOptionPane.showMessageDialog( SubsetWindow.this, msg,
-                                                   "Expression Syntax Error",
-                                                   JOptionPane.ERROR_MESSAGE );
-                    return;
-                }
-                subsetCounts.remove( rset );
-                subsets.set( irow, rset );
-                tcModel.getViewModel().fireTableDataChanged();
             }
         };
 
@@ -477,7 +458,7 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
      * MetaColumnTableModel displayed in this window.
      *
      * @param   irow  index into subsets list (unsorted table model)
-     * @return  subset at <tt>irow</tt>
+     * @return  subset at <code>irow</code>
      */
     private RowSubset getSubset( int irow ) {
         if ( irow < subsets.size() ) {
@@ -598,6 +579,10 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
     }
     public void intervalAdded( ListDataEvent evt ) {
         subsetsTableModel.fireTableDataChanged();
+        int irow = evt.getIndex0();
+        if ( irow >= 0 && irow == evt.getIndex1() ) {
+            TopcatUtils.ensureRowIndexIsVisible( jtab, irow );
+        }
     }
     public void intervalRemoved( ListDataEvent evt ) {
         subsetsTableModel.fireTableDataChanged();
@@ -614,7 +599,7 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
         public void actionPerformed( ActionEvent evt ) {
             Component parent = SubsetWindow.this;
             if ( this == addAct ) {
-                new SyntheticSubsetQueryWindow( tcModel, parent )
+                SubsetQueryWindow.newSubsetDialog( tcModel, parent )
                .setVisible( true );
             }
 
@@ -631,7 +616,8 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
 
             else if ( this == tocolAct ) {
                 SyntheticColumnQueryWindow colwin =
-                    new SyntheticColumnQueryWindow( tcModel, -1, parent );
+                    SyntheticColumnQueryWindow
+                   .newColumnDialog( tcModel, -1, parent );
                 int irow = toUnsortedIndex( jtab.getSelectedRow() );
                 colwin.setExpression( getSubsetID( irow ) );
                 colwin.setName( getSubsetName( irow ) );
@@ -652,6 +638,13 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
                 sc.start();
             }
 
+            else if ( this == editAct ) {
+                int irow = toUnsortedIndex( jtab.getSelectedRow() );
+                SubsetQueryWindow
+               .editSubsetDialog( tcModel, SubsetWindow.this, irow )
+               .setVisible( true );
+            }
+
             else if ( this == invertAct ) {
                 int irow = toUnsortedIndex( jtab.getSelectedRow() );
                 subsets.add( new InverseRowSubset( getSubset( irow ) ) );
@@ -790,7 +783,7 @@ public class SubsetWindow extends AuxWindow implements ListDataListener {
                         /* Update the subset counts. */
                         for ( int i = 0; i < nrset; i++ ) {
                             subsetCounts.put( rsets[ i ],
-                                              new Long( counts[ i ] ) );
+                                              Long.valueOf( counts[ i ] ) );
                         }
 
                         /* Notify listeners that the counts have changed. */
