@@ -1,5 +1,6 @@
 package uk.ac.starlink.topcat;
 
+import java.awt.Component;
 import java.awt.Container;
 import java.awt.Desktop;
 import java.awt.Dimension;
@@ -10,6 +11,9 @@ import java.awt.datatransfer.StringSelection;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
+import java.lang.reflect.InvocationHandler;
+import java.lang.reflect.Method;
+import java.lang.reflect.Proxy;
 import java.net.HttpURLConnection;
 import java.net.URL;
 import java.net.URLConnection;
@@ -57,8 +61,6 @@ import uk.ac.starlink.util.URLUtils;
  */
 public class TopcatUtils {
 
-    private static Boolean canSog_;
-    private static Boolean canExec_;
     private static Boolean canJel_;
     private static String[] about_;
     private static String version_;
@@ -189,13 +191,13 @@ public class TopcatUtils {
 
     /**
      * Returns the base name of a column; that is one without any 
-     * suffix based on <tt>baseSuffix</tt>.
+     * suffix based on <code>baseSuffix</code>.
      * This method is used in conjunction with {@link #getDistinctName}.
      *
      * @param  origName  full name, possibly including bits of suffix
      * @param  baseSuffix   the base suffix string
      * @return  name without any suffix-like elements of the sort 
-     *          specified by <tt>baseSuffix</tt>
+     *          specified by <code>baseSuffix</code>
      */
     public static String getBaseName( String origName, String baseSuffix ) {
         return suffixPattern( baseSuffix )
@@ -206,15 +208,15 @@ public class TopcatUtils {
     /**
      * Returns a column name based on a given one which is guaranteed 
      * distinct from any others in the column list.
-     * If the submitted <tt>origName</tt> is already unique, it may be
+     * If the submitted <code>origName</code> is already unique, it may be
      * returned.  Otherwise a new name may be made which involves appending
-     * the given <tt>baseSuffix</tt> to it.
+     * the given <code>baseSuffix</code> to it.
      *
      * @param  colList  column list within which distinct naming is required
      * @param  origName  initial name
      * @param  baseSuffix  suffix used for deduplication
-     * @return  a name resembling <tt>origName</tt> which is not the same
-     *          as any existing column names in <tt>colList</tt>
+     * @return  a name resembling <code>origName</code> which is not the same
+     *          as any existing column names in <code>colList</code>
      * @see  #getBaseName
      */
     public static String getDistinctName( ColumnList colList, String origName, 
@@ -303,7 +305,6 @@ public class TopcatUtils {
                 "STIL Version " + getSTILVersion(),
                 "Starjava revision: " + getRevision(),
                 "JVM: " + InvokeUtils.getJavaVM(),
-                "SoG: " + ( canSog() ? "available" : "absent" ),
                 "",
                 "Author: Mark Taylor (Bristol University)",
                 "WWW: http://www.starlink.ac.uk/topcat/",
@@ -465,91 +466,21 @@ public class TopcatUtils {
         } );
     }
 
-    /**
-     * Indicates whether there are enough classes to make SoG work at runtime.
-     *
-     * @return  true iff it's safe to use a SoG-based viewer
-     */
-    public static boolean canSog() {
-        if ( canSog_ == null ) {
-            synchronized ( TopcatUtils.class ) {
-                try {
-
-                    /* Check for SOG classes themselves. */
-                    Class.forName( "uk.ac.starlink.sog.SOG" );
-
-                    /* Check for JAI.  Use this class because it's lightweight
-                     * and won't cause a whole cascade of other classes
-                     * to be loaded. */
-                    Class.forName( "javax.media.jai.util.CaselessStringKey" );
-
-                    /* If we've got this far, we're OK. */
-                    canSog_ = Boolean.TRUE;
-                }
-                catch ( Throwable th ) {
-                    logger_.info( "No SoG: " + th );
-                    logger_.log( Level.CONFIG, "SoG load error", th );
-                    canSog_ = Boolean.FALSE;
-                }
-            }
-        }
-        return canSog_.booleanValue();
-    }
-
-    /**
-     * Indicates whether we have System.exec permission.
-     * 
-     * @return  true  if it's possible to exec
-     */
-    public static boolean canExec() {
-        if ( canExec_ == null ) {
-            synchronized ( TopcatUtils.class ) {
-                SecurityManager sman = System.getSecurityManager();
-                if ( sman != null ) {
-                    try {
-                        sman.checkExec( null );
-                    }
-                    catch ( SecurityException e ) {
-                        logger_.warning( "Security manager forbids " +
-                                         "system execution" );
-                        canExec_ = Boolean.FALSE;
-                    }
-                }
-                canExec_ = Boolean.TRUE;
-            }
-        }
-        return canExec_.booleanValue();
-    }
-
     /**
      * Indicates if it's possible to use JEL to compile algebraic expressions.
-     * If the security manager does not permit creation of private
-     * class loaders, it will fail.
      *
-     * @return   true iff JEL epxression compilation will work
+     * <p>This used to query the SecurityManager, but the
+     * <code>java.lang.SecurityManager</code> class is
+     * deprecated for removal in later JREs, and I don't think it's
+     * likely that expression compilation will be blocked.
+     * So this method currently returns true unconditionally,
+     * but retain it in case it turns out there are circumstances
+     * we can test for that bear on this.
+     *
+     * @return   true iff JEL epxression compilation is expected to work
      */
     public static boolean canJel() {
-        if ( canJel_ == null ) {
-            boolean can;
-            synchronized ( TopcatUtils.class ) {
-                SecurityManager sman = System.getSecurityManager();
-                if ( sman != null ) {
-                    try {
-                        sman.checkCreateClassLoader();
-                        can = true;
-                    }
-                    catch ( SecurityException e ) {
-                        logger_.warning( "Security manager forbids JEL use" );
-                        can = false;
-                    }
-                }
-                else {
-                    can = true;
-                }
-            }
-            canJel_ = Boolean.valueOf( can );
-        }
-        return canJel_.booleanValue();
+        return true;
     }
 
     /**
@@ -715,6 +646,97 @@ public class TopcatUtils {
         }
     }
 
+    /**
+     * Displays an informational About TOPCAT window.
+     *
+     * @param  parent  parent component
+     */
+    public static void showAbout( Component parent ) {
+        Object[] msg = new Object[] {
+            TopcatUtils.getAbout(),
+            Box.createVerticalStrut( 10 ),
+            AuxWindow.getSponsorLogos(),
+        };
+        JOptionPane.showMessageDialog( parent, msg, "About TOPCAT",
+                                       JOptionPane.INFORMATION_MESSAGE,
+                                       ResourceIcon.getTopcatLogo() );
+    }
+
+    /**
+     * Invokes the java.awt.Desktop.setAboutHandler method.
+     * Since that method was introduced at Java 9, and at time of writing
+     * the build and minimal target deployment platform is Java 8,
+     * this is done by reflection, and hence will only do useful work
+     * in a runtime environment of &gt;=9.  
+     * This method only does useful work anyway on certain platforms,
+     * notably MacOS, for which an About button is part of the standard
+     * application GUI.
+     * If for whatever reason the assignment doesn't work, it fails silently.
+     *
+     * <p>If the target platform is upgraded one day to Java &gt;= 9,
+     * this reflective code can be eliminated or replaced by something
+     * very much simpler.
+     *
+     * @param  doAbout  callback to display an About Application window
+     * @return  true iff the handler installation apparently succeeded
+     */
+    public static boolean setAboutHandler( Runnable doAbout ) {
+
+        /* See if the AboutHandler interface exists.
+         * If not, we're using a java version <9, so none of this reflection
+         * will work; bail out. */
+        final Class<?> aboutHandlerClazz;
+        try {
+            aboutHandlerClazz =
+                Class.forName( "java.awt.desktop.AboutHandler" );
+        }
+        catch ( Throwable e ) {
+            return false;
+        }
+        try {
+
+            /* Create an AboutHandler instance whose only method (handleAbout)
+             * runs the supplied callback code.  This has to be a proxy. */
+            InvocationHandler handler = new InvocationHandler() {
+                public Object invoke( Object proxy, Method method,
+                                      Object[] args ) {
+                    if ( "handleAbout".equals( method.getName() ) ) {
+                        doAbout.run();
+                    }
+                    return null;
+                }
+            };
+            Object aboutHandler =
+                Proxy.newProxyInstance( TopcatUtils.class.getClassLoader(),
+                                        new Class<?>[] { aboutHandlerClazz },
+                                        handler );
+
+            /* Locate and invoke the Desktop.setAboutHandler method
+             * reflectively. */
+            Method setAboutHandlerMethod =
+                Arrays.stream( Desktop.class.getMethods() )
+                      .filter( m -> "setAboutHandler".equals( m.getName() ) )
+                      .findAny()
+                      .get();
+            setAboutHandlerMethod.invoke( Desktop.getDesktop(), aboutHandler );
+            return true;
+        }
+
+        /* setAboutHandler fails with an UnsupportedOperationException on
+         * non-Mac platforms. */
+        catch ( UnsupportedOperationException e ) {
+            return false;
+        }
+
+        /* Otherwise it looks like we're on Java 9, but something went wrong,
+         * probably with the reflection.  Oh well. */
+        catch ( Throwable e ) {
+            logger_.log( Level.INFO,
+                         "Slightly surprising reflection failure", e );
+            return false;
+        }
+    }
+
     /**
      * Using input from the user, adds a new (or reused) Row Subset
      * to the given TopcatModel based on a given BitSet.
@@ -834,7 +856,8 @@ public class TopcatUtils {
         IOException e1 = null;
         try {
             URLConnection conn =
-                URLUtils.followRedirects( new URL( url ).openConnection(),
+                URLUtils.followRedirects( URLUtils.newURL( url )
+                                                  .openConnection(),
                                           (int[]) null );
             if ( conn instanceof HttpURLConnection &&
                  ((HttpURLConnection) conn).getResponseCode() == 200 ) {
@@ -851,7 +874,7 @@ public class TopcatUtils {
             try {
                 URLConnection conn2 =
                     URLUtils
-                   .followRedirects( new URL( url2 ).openConnection(),
+                   .followRedirects( URLUtils.newURL( url2 ).openConnection(),
                                      (int[]) null );
                 if ( conn2 instanceof HttpURLConnection &&
                      ((HttpURLConnection) conn2).getResponseCode() == 200 ) {
